#include <math.h>
#include <stdio.h>
#include <stdlib.h>

int usage() {
	printf("ppm84cse [PPM file name] [data label]\n");
	printf("\n");
	printf("This software is intended to convert PPM image files\n");
	printf("into Z80 assembly in order to be used on the TI-84+CSE\n");
	printf("graphing calculator.\n");
	printf("\n");
	printf("As PPM is recognized by ImageMagick, other image\n");
	printf("formats can be easily converted to PPM to be used with\n");
	printf("this software.\n");
	printf("\n");
	printf("Usage:\n");
	printf("	convert dog.png dog.ppm\n");
	printf("	ppm84cse dog.ppm DOGIMG > dog.asm\n");
	printf("\n");
	printf("The data label is added the top of the assembly file\n");
	printf("for locating it in your program.\n\n");
}

int RGB(double r, double g, double b) {

	r = floor(r/8);
	g = floor(g/4);
	b = floor(b/8);

	double rgb = r*pow(2,11) + g*pow(2,5) + b ;
	return (int)rgb;
}

int main(int argc, char** args) {

	if (argc < 3) { 
		usage();
		return 1;
	}
	FILE *file = fopen(args[1],"rb");
	if (file == NULL) {
		usage();
		return 1;
	}

	fseek(file,0,SEEK_END);
	int size=ftell(file);
	rewind(file);

	unsigned char buffer[size];
	fread(buffer,size,1,file);
	
	int dataSectionAddr=0;
	int lineCount=0;
	int width=-1;
	int height=-1;
	while (lineCount<3) {
		if (buffer[dataSectionAddr] == (int)'#') {
			lineCount--;
		}

		if (lineCount == 1 && width==-1 && height==-1) {
			int sizeOfFirst = 0;
			int sizeOfSecond = 0;
			while (buffer[dataSectionAddr+sizeOfFirst] != (char)' ') 
				sizeOfFirst++;
			while (buffer[dataSectionAddr+sizeOfFirst+sizeOfSecond+1] != (char)'\n') 
				sizeOfSecond++;

			char first[sizeOfFirst];
			char second[sizeOfSecond];

			for (int i=0; i<sizeOfFirst; i++)
				first[i] = buffer[dataSectionAddr+i];
			for (int i=0; i<sizeOfSecond; i++)
				second[i] = buffer[dataSectionAddr+sizeOfFirst+1+i];

			width = atoi(first);
			height = atoi(second);

		}

		if (buffer[dataSectionAddr] == (int)'\n') 
			lineCount++;
		
		dataSectionAddr++;
	}

	printf("%s:\n\t.dw ",args[2]);

	int d = width;
	for (int i=dataSectionAddr; i<size; i+=3) {
		int rgb = RGB(buffer[i],buffer[i+1],buffer[i+2]);
		printf("$%x",rgb);

		d--;
		if (i+3 < size)
			printf(", ");
	}
	printf("\n\n");

}
